<!-- Start Datatables-js from cdnjs -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/datatables.net/2.1.6/dataTables.min.js" integrity="sha512-CxrfZ72a/ewXZ7NNP2qyfJeFc7/+snLwdhbX1Ops4FSyCHnnwMsB9OOJfQagb+neE5yGpj/ExNPRaUkJGE+zhg==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/datatables.net-bs5/2.1.6/dataTables.bootstrap5.min.js" integrity="sha512-Cwi0jz7fz7mrX990DlJ1+rmiH/D9/rjfOoEex8C9qrPRDDqwMPdWV7pJFKzhM10gAAPlufZcWhfMuPN699Ej0w==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
<!-- End Datatables-js from cdnjs -->
