# Generated by Django 1.11.5 on 2017-09-28 02:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('srp', '0003_make_strings_more_stringy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_commander',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.EveCharacter'),
        ),
        migrations.AlterField(
            model_name='srpuserrequest',
            name='character',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.EveCharacter'),
        ),
    ]
